/**
  ******************************************************************************
  * @file    Project/STM32L1xx_StdPeriph_Template/main.c
  * @author  MCD Application Team
  * @version V1.0.0RC1
  * @date    07/02/2010
  * @brief   Main program body
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "includes.h"

/** @addtogroup Template_Project
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#ifdef USE_STM32L152_EVAL
  #define MESSAGE1   "  STM32L Low Power  "
  #define MESSAGE2   " Device running on  "
  #define MESSAGE3   "   STM32L152-EVAL   "
#endif


/* Private macro -------------------------------------------------------------*/
#define ADC1_DR_ADDRESS    ((uint32_t)0x40012458)

/* Private variables ---------------------------------------------------------*/
USART_InitTypeDef USART_InitStructure;
static __IO uint32_t TimingDelay;
RCC_ClocksTypeDef RCC_Clocks;
__IO uint16_t ADC_ConvertedValues[2];
uint32_t CriticalSecCntr;
volatile uint32_t CTS_Change;
volatile uint32_t Butt_Change;

/* Private function prototypes -----------------------------------------------*/
#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
   set to 'Yes') calls __io_putchar() */
#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#define GETCHAR_PROTOTYPE int fgetc(FILE *f)
#endif /* __GNUC__ */

uint8_t LCD_String[8]= " STM32L ";
#define SCROLL_SPEED  5000 /* Low value gives higher speed */
#define SCROLL_NUM    20

// this function enables or disables the pin that sinks the battery resistor bridge
void EnableBatteryMeasurement(Boolean enable);
// this function enables or disables the pin that sinks the potentiometer pin
void EnablePotentiometer(Boolean enable);
// this function enables or disables the pin that controls RS232 chip power supply
void EnableRS232Chip(Boolean enable);

void EnterDeepSleepMode(void);


/* Private functions ---------------------------------------------------------*/

void RCC_Configuration(void)
{
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

  /* Allow access to the RTC */
  PWR_RTCAccessCmd(ENABLE);

  /* Reset Backup Domain */
  RCC_RTCResetCmd(ENABLE);
  RCC_RTCResetCmd(DISABLE);

  /*!< LSE Enable */
  RCC_LSEConfig(RCC_LSE_ON);

  /*!< Wait till LSE is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {}
  /*!< LCD Clock Source Selection */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
}

#define SEG_MASK(seg)     (seg & (32-1))
#define SEG_EL(seg,com)   (volatile unsigned int *)(PERIPH_BASE + 0x2000000 + ((0x2400 + 0x14 + ((com*2) + ((seg<32)?0:1))*4)*32) + (SEG_MASK(seg)*4))

#define size_of(data) (sizeof(data)/sizeof(data[0]))

volatile unsigned int * Table[] =
{
  SEG_EL(39,3), // +    0
  SEG_EL(39,0), // -    1
  SEG_EL(37,3), // 1A   2
  SEG_EL(37,2), // 1B   3
  SEG_EL(37,1), // 1C   4
  SEG_EL(37,0), // 1D   5
  SEG_EL(39,1), // 1E   6
  SEG_EL(39,2), // 1F   7
  SEG_EL(38,2), // 1G   8
  SEG_EL(38,3), // 1H   9
  SEG_EL(36,3), // 1J   10
  SEG_EL(36,2), // 1K   11
  SEG_EL(36,1), // 1M   12
  SEG_EL(36,0), // 1N   13
  SEG_EL(38,0), // 1P   14
  SEG_EL(38,1), // 1Q   15
  SEG_EL(35,0), // 1DP  16
  SEG_EL(35,3), // 2COL 17

  SEG_EL(33,3), // 2A   18
  SEG_EL(33,2), // 2B   19
  SEG_EL(33,1), // 2C   20
  SEG_EL(33,0), // 2D   21
  SEG_EL(35,1), // 2E   22
  SEG_EL(35,2), // 2F   23
  SEG_EL(34,2), // 2G   24
  SEG_EL(34,3), // 2H   25
  SEG_EL(32,3), // 2J   26
  SEG_EL(32,2), // 2K   27
  SEG_EL(32,1), // 2M   28
  SEG_EL(32,0), // 2N   29
  SEG_EL(34,0), // 2P   30
  SEG_EL(34,1), // 2Q   31
  SEG_EL(31,0), // 2DP  32
  SEG_EL(31,3), // 3COL 33

  SEG_EL(29,3), // 3A   34
  SEG_EL(29,2), // 3B   35
  SEG_EL(29,1), // 3C   36
  SEG_EL(29,0), // 3D   37
  SEG_EL(31,1), // 3E   38
  SEG_EL(31,2), // 3F   39
  SEG_EL(30,2), // 3G   40
  SEG_EL(30,3), // 3H   41
  SEG_EL(28,3), // 3J   42
  SEG_EL(28,2), // 3K   43
  SEG_EL(28,1), // 3M   44
  SEG_EL(28,0), // 3N   45
  SEG_EL(30,0), // 3P   46
  SEG_EL(30,1), // 3Q   47
  SEG_EL(27,0), // 3DP  48
  SEG_EL(27,3), // 4COL 49

  SEG_EL(25,3), // 4A   50
  SEG_EL(25,2), // 4B   51
  SEG_EL(25,1), // 4C   52
  SEG_EL(25,0), // 4D   53
  SEG_EL(27,1), // 4E   54
  SEG_EL(27,2), // 4F   55
  SEG_EL(26,2), // 4G   56
  SEG_EL(26,3), // 4H   57
  SEG_EL(24,3), // 4J   58
  SEG_EL(24,2), // 4K   59
  SEG_EL(24,1), // 4M   60
  SEG_EL(24,0), // 4N   61
  SEG_EL(26,0), // 4P   62
  SEG_EL(26,1), // 4Q   63
  SEG_EL(23,0), // 4DP  64
  SEG_EL(23,3), // 5COL 65

  SEG_EL(21,3), // 5A   66
  SEG_EL(21,2), // 5B   67
  SEG_EL(21,1), // 5C   68
  SEG_EL(21,0), // 5D   69
  SEG_EL(23,1), // 5E   70
  SEG_EL(23,2), // 5F   71
  SEG_EL(22,2), // 5G   72
  SEG_EL(22,3), // 5H   73
  SEG_EL(20,3), // 5J   74
  SEG_EL(20,2), // 5K   75
  SEG_EL(20,1), // 5M   76
  SEG_EL(20,0), // 5N   77
  SEG_EL(22,0), // 5P   78
  SEG_EL(22,1), // 5Q   79
  SEG_EL(19,0), // 5DP  80
  SEG_EL(19,3), // 6COL 81

  SEG_EL(17,3), // 6A   82
  SEG_EL(17,2), // 6B   83
  SEG_EL(17,1), // 6C   84
  SEG_EL(17,0), // 6D   85
  SEG_EL(19,1), // 6E   86
  SEG_EL(19,2), // 6F   87
  SEG_EL(18,2), // 6G   88
  SEG_EL(18,3), // 6H   89
  SEG_EL(16,3), // 6J   90
  SEG_EL(16,2), // 6K   91
  SEG_EL(16,1), // 6M   92
  SEG_EL(16,0), // 6N   93
  SEG_EL(18,0), // 6P   94
  SEG_EL(18,1), // 6Q   95
  SEG_EL(15,0), // 6DP  96

  SEG_EL(13,3), // 7A   97
  SEG_EL(13,2), // 7B   98
  SEG_EL(13,1), // 7C   99
  SEG_EL(13,0), // 7D   100
  SEG_EL(15,1), // 7E   101
  SEG_EL(15,2), // 7F   102
  SEG_EL(14,2), // 7G   103
  SEG_EL(14,3), // 7H   104
  SEG_EL(12,3), // 7J   105
  SEG_EL(12,2), // 7K   106
  SEG_EL(12,1), // 7M   107
  SEG_EL(12,0), // 7N   108
  SEG_EL(14,0), // 7P   109
  SEG_EL(14,1), // 7Q   110

  SEG_EL(1 ,3), // A1   111
  SEG_EL(1 ,2), // A2   112
  SEG_EL(1 ,1), // A3   113
  SEG_EL(1 ,0), // A4   114

  SEG_EL(2 ,0), // BRBL 115
  SEG_EL(2 ,3), // B0   116
  SEG_EL(2 ,2), // B1   117
  SEG_EL(2 ,1), // B2   118

  SEG_EL(0 ,3), // PL   119
  SEG_EL(0 ,2), // P0   120
  SEG_EL(0 ,1), // P1   121
  SEG_EL(0 ,0), // P2   122
  SEG_EL(43,0), // P3   123
  SEG_EL(43,1), // P4   124
  SEG_EL(43,2), // P5   125
  SEG_EL(43,3), // P6   126
  SEG_EL(42,3), // P7   127
  SEG_EL(42,2), // P8   128
  SEG_EL(42,1), // P9   129
  SEG_EL(42,0), // PR   130

  SEG_EL(3 ,0), // AL   131
  SEG_EL(3 ,1), // AU   132
  SEG_EL(3 ,2), // AR   133
  SEG_EL(3 ,3), // AD   134

  SEG_EL(15,3), // SB   135

  SEG_EL(10,0), // 8A   136
  SEG_EL(10,1), // 8B   137
  SEG_EL(10,2), // 8C   138
  SEG_EL(11,3), // 8D   139
  SEG_EL(11,2), // 8E   140
  SEG_EL(11,0), // 8F   141
  SEG_EL(11,1), // 8G   142
  SEG_EL(10,3), // 8P   143

  SEG_EL(8 ,0), // 9A   144
  SEG_EL(8 ,1), // 9B   145
  SEG_EL(8 ,2), // 9C   146
  SEG_EL(9 ,3), // 9D   147
  SEG_EL(9 ,2), // 9E   148
  SEG_EL(9 ,0), // 9F   149
  SEG_EL(9 ,1), // 9G   150
  SEG_EL(8 ,3), // 10P  151
  SEG_EL(7 ,3), // 10COLON  152

  SEG_EL(7 ,0), // 10A  153
  SEG_EL(6 ,0), // 10B  154
  SEG_EL(6 ,2), // 10C  155
  SEG_EL(6 ,3), // 10D  156
  SEG_EL(7 ,2), // 10E  157
  SEG_EL(7 ,1), // 10F  158
  SEG_EL(6 ,1), // 10G  159

  SEG_EL(5 ,0), // 11A  160
  SEG_EL(4 ,0), // 11B  161
  SEG_EL(4 ,2), // 11C  162
  SEG_EL(4 ,3), // 11D  163
  SEG_EL(5 ,2), // 11E  164
  SEG_EL(5 ,1), // 11F  165
  SEG_EL(4 ,1), // 11G  166
};

#define SEG_A (1ul<<0)
#define SEG_B (1ul<<1)
#define SEG_C (1ul<<2)
#define SEG_D (1ul<<3)
#define SEG_E (1ul<<4)
#define SEG_F (1ul<<5)
#define SEG_G (1ul<<6)
#define SEG_H (1ul<<7)
#define SEG_J (1ul<<8)
#define SEG_K (1ul<<9)
#define SEG_M (1ul<<10)
#define SEG_N (1ul<<11)
#define SEG_P (1ul<<12)
#define SEG_Q (1ul<<13)

#define LARGE_SIZE (14)
#define SMALL_SIZE (7)

#define Digit_0 (SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F)
#define Digit_1 (SEG_B | SEG_C)
#define Digit_2 (SEG_A | SEG_B | SEG_G | SEG_E | SEG_D | SEG_M)
#define Digit_3 (SEG_A | SEG_B | SEG_C | SEG_D | SEG_G | SEG_M)
#define Digit_4 (SEG_F | SEG_B | SEG_C | SEG_G | SEG_M)
#define Digit_5 (SEG_A | SEG_F | SEG_G | SEG_C | SEG_D | SEG_M)
#define Digit_6 (SEG_A | SEG_F | SEG_G | SEG_C | SEG_D | SEG_E | SEG_M)
#define Digit_7 (SEG_A | SEG_B | SEG_C)
#define Digit_8 (SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G | SEG_M)
#define Digit_9 (SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G | SEG_M)

uint32_t dig2sym[] = {
  Digit_0, Digit_1, Digit_2, Digit_3, Digit_4,
  Digit_5, Digit_6, Digit_7, Digit_8, Digit_9
};

#define Large_Digit_0_offset  2
#define Large_Digit_1_offset  18
#define Large_Digit_2_offset  34
#define Large_Digit_3_offset  50
#define Large_Digit_4_offset  66
#define Large_Digit_5_offset  82
#define Large_Digit_6_offset  97

#define Small_Digit_0_offset  136
#define Small_Digit_1_offset  144
#define Small_Digit_2_offset  153
#define Small_Digit_3_offset  160

#define LeftArrow             131
#define RightArrow            133

void SetSeg(int State, volatile unsigned int * Seg)
{
  /*!< Wait Until the last LCD RAM update finish */
  while(LCD_GetFlagStatus(LCD_FLAG_UDR) != RESET)
  {
  }

  if(State)
    *Seg = 1;
  else
    *Seg = 0;
  /*!< Requesy LCD RAM update */
  LCD_UpdateDisplayRequest();
}


void SetAll(void)
{
  /*!< Wait Until the last LCD RAM update finish */
  while(LCD_GetFlagStatus(LCD_FLAG_UDR) != RESET)
  {
  }
  for(int i = 0; i < size_of(Table); i++ )
  {
    *Table[i] = 1;
  }
  /*!< Requesy LCD RAM update */
  LCD_UpdateDisplayRequest();
}

void ShowSymbol(uint32_t offset, uint32_t mask, uint32_t size)
{
  volatile unsigned int ** p_data = &Table[offset];

  while(LCD_GetFlagStatus(LCD_FLAG_UDR) != RESET)
  {
  }

  for(int i = 0, j = 1; i < size; i++, j <<= 1)
  {
    if(mask & j)
    {
      **p_data = 1;
    }
    else
    {
      **p_data = 0;
    }
    ++p_data;
  }

  /*!< Requesy LCD RAM update */
  LCD_UpdateDisplayRequest();
}

void ClearAll(void)
{
  /*!< Wait Until the last LCD RAM update finish */
  while(LCD_GetFlagStatus(LCD_FLAG_UDR) != RESET)
  {
  }
  for(int i = 0; i < size_of(Table); i++ )
  {
    *Table[i] &= 0;
  }
  /*!< Requesy LCD RAM update */
  LCD_UpdateDisplayRequest();
}



/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
GPIO_InitTypeDef GPIO_InitStructure;
ADC_InitTypeDef ADC_InitStructure;
DMA_InitTypeDef DMA_InitStructure;
EXTI_InitTypeDef EXTI_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;
uint32_t temp;
char ch;
uint32_t loop;
uint32_t dly_led1;
uint32_t dly_led2;
uint32_t dly_led3;
uint32_t dly_led4;
uint32_t adc_dly;

/*!< At this stage the microcontroller clock setting is already configured,
       this is done through SystemInit() function which is called from startup
       file (startup_stm32l1xx_md.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32l1xx.c file
     */


  /* SysTick end of count event each 10ms */
  RCC_GetClocksFreq(&RCC_Clocks);
  SysTick_Config(RCC_Clocks.HCLK_Frequency / 100);

  RCC_AHBPeriphResetCmd(  RCC_AHBPeriph_GPIOA
                         | RCC_AHBPeriph_GPIOB
                         | RCC_AHBPeriph_GPIOC
                         | RCC_AHBPeriph_GPIOD
                         | RCC_AHBPeriph_GPIOE,
                         DISABLE);
  RCC_AHBPeriphClockCmd(   RCC_AHBPeriph_GPIOA
                         | RCC_AHBPeriph_GPIOB
                         | RCC_AHBPeriph_GPIOC
                         | RCC_AHBPeriph_GPIOD
                         | RCC_AHBPeriph_GPIOE,
                         ENABLE);
  
#ifndef  USING_JTAG_MODE // SPP +
  // blocking program
  STM_EVAL_PBInit(BUTTON_WAKEUP,BUTTON_MODE_EXTI);
  if (STM_EVAL_PBGetState(BUTTON_WAKEUP))
    while (1)
#endif  // SPP -

  /* Initialize LEDs, Key Button, LCD and COM port(USART) available on
     STM32L1XX-EVAL board *****************************************************/

  /* USARTx configured as follow:
        - BaudRate = 115200 baud
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOD, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  GPIO_ResetBits(GPIOD,GPIO_Pin_0 | GPIO_Pin_1);
  GPIO_SetBits(GPIOD,GPIO_Pin_4);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  /* Connect Button EXTI Line to CTS (PD3)*/
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOD, EXTI_PinSource3);

  /* Configure Button EXTI line */
  EXTI_DeInit();
  EXTI_InitStructure.EXTI_Line = EXTI_Line3;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Enable and set Button EXTI Interrupt to the lowest priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI3_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);

  USART_InitStructure.USART_BaudRate = 115200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  STM_EVAL_COMInit(COM1, &USART_InitStructure);

  printf("\n\rSTM32-P152 production test program - %s, %s\n\r", __DATE__, __TIME__);

      
  STM_EVAL_PBInit(BUTTON_USER,BUTTON_MODE_EXTI);
  if( !STM_EVAL_PBGetState(BUTTON_USER) ) {
	  // UART Test
	  printf("UART test, press \"S\" for test skip\n\r");
	  ch = getchar();
	  if(   'S' != ch
		 && 's' != ch )
	  {
		printf("Toggle state of RTS line and check that CTS toggeled too\n\r");
		printf("Press any key to continue\n\r");
		while(1)
		{
		  if(USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) != RESET)
		  {
			ch = getchar();
			break;
		  }
		  if(CTS_Change)
		  {
			CTS_Change = 0;
			GPIO_ResetBits(GPIOD,GPIO_Pin_4);
			Delay(35);
			GPIO_SetBits(GPIOD,GPIO_Pin_4);
		  }
		}
	  }
	
	  STM_EVAL_LEDInit(LED1);
	  STM_EVAL_LEDInit(LED2);
	  STM_EVAL_LEDInit(LED3);
	  STM_EVAL_LEDInit(LED4);
	  STM_EVAL_LEDOn(LED1);
	  STM_EVAL_LEDOn(LED2);
	
#ifndef  USING_JTAG_MODE // SPP +
	  // LCD Test
	  printf("Init RTC ...\n\r");
	  /* Initialize the LCD */
	  RCC_Configuration();
	  /* LCD GLASS Initialization */
	  LCD_GLASS_Init();
	  ClearAll();
	
	  printf("LCD test, press \"S\" for test skip\n\r");
	  ch = getchar();
	  if(   'S' != ch
		 && 's' != ch )
	  {
		printf("LCD test 1, press any key to continue\n\r");
		loop = 1;
		while(loop)
		{
		  for(int i = 0; i < size_of(Table); i++)
		  {
			SetSeg(1,Table[i]);
			Delay(15);
			SetSeg(0,Table[i]);
			if(USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) != RESET)
			{
			  ch = getchar();
			  loop = 0;
			  break;
			}
		  }
		}
	
		printf("LCD test 2, press any key to continue\n\r");
		loop = 1;
		while(loop)
		{
		  SetAll();
		  Delay(25);
		  ClearAll();
		  Delay(25);
		  if(USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) != RESET)
		  {
			ch = getchar();
			loop = 0;
		  }
		}
		ClearAll();
	  }
#endif  // SPP -

	  // USB test
	  printf("USB test, press \"S\" for test skip\n\r");
	  ch = getchar();
	  if(   'S' != ch
		 && 's' != ch )
	  {
		// HID USB
		HidInit();
		// Soft connection enable
		USB_ConnectRes(TRUE);
	
		printf("Is mouse pointer move, press any key to continue\n\r");
	
		loop = 1;
		while(loop)
		{
		  Delay(25);
		  HidMouseSendReport(-20,0,0);
		  Delay(25);
		  HidMouseSendReport( 20,0,0);
		  if(USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) != RESET)
		  {
			ch = getchar();
			loop = 0;
		  }
		}
		USB_ConnectRes(FALSE);
	  }
  }

  // Buttons, LEDS and Trimmer test
#ifndef  USING_JTAG_MODE // SPP +
  /* LCD GLASS Initialization */
  RCC_Configuration();
  LCD_GLASS_Init();
#endif  // SPP -

  /* Buttons init*/
  STM_EVAL_PBInit(BUTTON_WAKEUP,BUTTON_MODE_EXTI);
  STM_EVAL_PBInit(BUTTON_USER,BUTTON_MODE_EXTI);

  EnableBatteryMeasurement(TRUE);

  /* ADC Init */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1 | RCC_AHBPeriph_GPIOE, ENABLE);
  /* Enable ADC1 clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
  /* Enable the HSI */
  RCC_HSICmd(ENABLE);

  GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  /* DMA1 channel1 configuration ---------------------------------------------*/
  DMA_DeInit(DMA1_Channel1);
  DMA_InitStructure.DMA_PeripheralBaseAddr = ADC1_DR_ADDRESS;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&ADC_ConvertedValues[0];
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
  DMA_InitStructure.DMA_BufferSize = 2;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);

  /* Enable DMA1 channel1 */
  DMA_Cmd(DMA1_Channel1, ENABLE);

  /* ADC1 configuration ------------------------------------------------------*/
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b;
  ADC_InitStructure.ADC_ScanConvMode = ENABLE;
//  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_ExternalTrigConv = 0;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  ADC_InitStructure.ADC_NbrOfConversion = 2;
  ADC_Init(ADC1, &ADC_InitStructure);
  ADC_DelaySelectionConfig(ADC1, ADC_DelayLength_255Cycles); // insert delay between conversions

  /* ADC1 regular channel18 configuration */
  ADC_RegularChannelConfig(ADC1, ADC_Channel_22, 1, ADC_SampleTime_4Cycles); // this is the potentiometer
  ADC_RegularChannelConfig(ADC1, ADC_Channel_23, 2, ADC_SampleTime_4Cycles); // this is the battery measurement

  /* Enable the request after last transfer for DMA Circular mode */
  ADC_DMARequestAfterLastTransferCmd(ADC1, ENABLE);

  /* Enable ADC1 DMA */
  ADC_DMACmd(ADC1, ENABLE);

  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);

  /* Start ADC1 Software Conversion */
  ADC_SoftwareStartConv(ADC1);

  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  STM_EVAL_LEDInit(LED3);
  STM_EVAL_LEDInit(LED4);
  STM_EVAL_LEDOff(LED1);
  STM_EVAL_LEDOff(LED2);
  STM_EVAL_LEDOn(LED3);
  STM_EVAL_LEDOn(LED4);

  printf("Check buttons, LEDs and trimmer\n\r");
  dly_led1 = TimingDelay + 25;
  dly_led2 = dly_led1 + 25;
  dly_led3 = dly_led2 + 25;
  dly_led4 = dly_led3 + 25;
  adc_dly = TimingDelay + 25;


  while(1)
  {
    temp = AtomicExchange(0,(uint32_t*)&Butt_Change);
    if(temp)
    {
      Butt_Change = 0;
#ifndef  USING_JTAG_MODE // SPP +
      if(STM_EVAL_PBGetState(BUTTON_WAKEUP))
      {
        SetSeg(1, Table[RightArrow]);
      }
      else
      {
        SetSeg(0, Table[RightArrow]);
      }
      if(!STM_EVAL_PBGetState(BUTTON_USER))
      {
        SetSeg(1, Table[LeftArrow]);
		
		// if the button is pressed disable the output that sinks the resistro bridge for battery measurement
//		EnableBatteryMeasurement(FALSE);
		
		EnterDeepSleepMode();		
      }
      else
      {
        SetSeg(0, Table[LeftArrow]);
      }
#else
      if(!STM_EVAL_PBGetState(BUTTON_USER))
		EnterDeepSleepMode();	
#endif  // SPP -
    }

    if(dly_led1 < TimingDelay)
    {
      dly_led1 = dly_led4 + 25;
      STM_EVAL_LEDToggle(LED1);
    }
    if(dly_led2 < TimingDelay)
    {
      dly_led2 = dly_led1 + 25;
      STM_EVAL_LEDToggle(LED2);
    }
    if(dly_led3 < TimingDelay)
    {
      dly_led3 = dly_led2 + 25;
      STM_EVAL_LEDToggle(LED3);
    }
    if(dly_led4 < TimingDelay)
    {
      dly_led4 = dly_led3 + 25;
      STM_EVAL_LEDToggle(LED4);
    }

    if(adc_dly < TimingDelay)
    {
      char str[10];
		
      adc_dly += 100;
	
	  // display result of the potentiometer measurement
      temp = (ADC_ConvertedValues[0] * 100)/(4096-1);
	
	  if(temp > 99)
		temp = 99;
	
      sprintf(str, "%02d", temp);

	  ShowSymbol(Large_Digit_0_offset,dig2sym[str[0] - '0'],LARGE_SIZE);
      ShowSymbol(Large_Digit_1_offset,dig2sym[str[1] - '0'],LARGE_SIZE);
	
	  // now display the result of the battery measurement
      temp = (ADC_ConvertedValues[1] * 14190)/(4096-1);

      sprintf(str, "%04d", temp);

      ShowSymbol(Large_Digit_3_offset,dig2sym[str[0] - '0'],LARGE_SIZE);
      ShowSymbol(Large_Digit_4_offset,dig2sym[str[1] - '0'],LARGE_SIZE);
      ShowSymbol(Large_Digit_5_offset,dig2sym[str[2] - '0'],LARGE_SIZE);
      ShowSymbol(Large_Digit_6_offset,dig2sym[str[3] - '0'],LARGE_SIZE);
    }
	
	 /* Start ADC1 Software Conversion */
	 ADC_SoftwareStartConv(ADC1);	
  }
}

/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
  USART_SendData(EVAL_COM1, (uint8_t) ch);

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TC) == RESET)
  {}

  return ch;
}

GETCHAR_PROTOTYPE
{
int ch;
  /* Place your implementation of fgetc here */
  /* e.g. read a character from the USART */

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) == RESET)
  {}

  ch = USART_ReceiveData(EVAL_COM1);
  return ch;
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(uint32_t nTime)
{
  nTime += TimingDelay;
  while(TimingDelay < nTime);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  ++TimingDelay;
}

// this function enables or disables the pin that sinks the battery resistor bridge
void EnableBatteryMeasurement(Boolean enable)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	if(enable) {
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		GPIO_Init(GPIOD, &GPIO_InitStructure);
		GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_RESET);
	} else {
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		GPIO_Init(GPIOD, &GPIO_InitStructure);
	}
}

// this function enables or disables the pin that sinks the potentiometer pin
void EnablePotentiometer(Boolean enable)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	if(enable) {
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		GPIO_Init(GPIOD, &GPIO_InitStructure);
		GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_RESET);
	} else {
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		GPIO_Init(GPIOD, &GPIO_InitStructure);
	}
}

// this function enables or disables the pin that controls RS232 chip power supply
void EnableRS232Chip(Boolean enable)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	
	if(enable) {
		GPIO_WriteBit(GPIOD, GPIO_Pin_0, Bit_RESET);
	} else {
		GPIO_WriteBit(GPIOD, GPIO_Pin_0, Bit_SET);
	}
}

void EnterDeepSleepMode(void)
{
	// turn off stuff
	EnableRS232Chip(FALSE);
	EnablePotentiometer(FALSE);
	EnableBatteryMeasurement(FALSE);
	
	// set pins to low power values
	
	// now disable power domain
	PWR_EnterSTANDBYMode();
	
	while(1);
}


#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/******************* (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE****/
